@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+import os
 import sys
 import argparse
 import logging
@@ -7,6 +8,7 @@ import logging.config
 from rq import Queue, Worker
 from rq.logutils import setup_loghandlers
 from redis.exceptions import ConnectionError
+from rq.contrib.legacy import cleanup_ghosts
 from rq.scripts import add_standard_arguments
 from rq.scripts import setup_redis
 from rq.scripts import read_config_file
@@ -23,12 +25,28 @@ def parse_args():
     parser.add_argument('--name', '-n', default=None, help='Specify a different name')
     parser.add_argument('--path', '-P', default='.', help='Specify the import path.')
     parser.add_argument('--verbose', '-v', action='store_true', default=False, help='Show more output')
+    parser.add_argument('--quiet', '-q', action='store_true', default=False, help='Show less output')
     parser.add_argument('--sentry-dsn', action='store', default=None, metavar='URL', help='Report exceptions to this Sentry DSN')
+    parser.add_argument('--pid', action='store', default=None,
+                        help='Write the process ID number to a file at the specified path')
     parser.add_argument('queues', nargs='*', help='The queues to listen on (default: \'default\')')
 
     return parser.parse_args()
 
 
+def setup_loghandlers_from_args(args):
+    if args.verbose and args.quiet:
+        raise RuntimeError("Flags --verbose and --quiet are mutually exclusive.")
+
+    if args.verbose:
+        level = 'DEBUG'
+    elif args.quiet:
+        level = 'WARNING'
+    else:
+        level = 'INFO'
+    setup_loghandlers(level)
+
+
 def main():
     args = parse_args()
 
@@ -41,13 +59,23 @@ def main():
 
     setup_default_arguments(args, settings)
 
-    # Other default arguments
-    if args.sentry_dsn is None:
-        args.sentry_dsn = settings.get('SENTRY_DSN', None)
+    # Worker specific default arguments
+    if not args.queues:
+        args.queues = settings.get('QUEUES', ['default'])
 
-    setup_loghandlers(args.verbose)
+    if args.sentry_dsn is None:
+        args.sentry_dsn = settings.get('SENTRY_DSN',
+                                       os.environ.get('SENTRY_DSN', None))
+
+    if args.pid:
+        with open(os.path.expanduser(args.pid), "w") as fp:
+            fp.write(str(os.getpid()))
+
+    setup_loghandlers_from_args(args)
     setup_redis(args)
 
+    cleanup_ghosts()
+
     try:
         queues = map(Queue, args.queues)
         w = Worker(queues, name=args.name)
